#!/usr/bin/perl


#	bluePlaylist
	$VERSION = 1.1;
#
#	(c) 2003 Thomas Unterthiner
#	generates a (X)HTML file from a .m3u playlist containing all its titles
#
################################################################################
#
# This script is free software; you can redistribute it and/or modify it under 
# the terms of the GNU General Public License as published by the Free Software 
# Foundation; either version 2 of the License, or (at your option) any later 
# version.
#
################################################################################
#
#	CHANGES:
#
#	2003/12/29
#	* fixed small typo
#	* added writeCopyrightNote()
#
#	2003/12/21
#	* if playlist couldn't be opened, show last OS-Error ($!) when die()-ing
#	* added GPL-info both in source code and script-output
#	* renamned parameter 'titles' to 'artists'
#	* restructured the whole code, split it up in functions
#	* commented each function
#	2003/12/10
#	* added possibility to search for all artists who have a given number of
#	  titles in the playlist (the -titles <N> - parameter)
#	2003/12/01
#	* added interpret-counting (output written to shell only)
#
################################################################################
#
#	USAGE:
#
# playlist.pl PLAYLISTFILE [OUTPUTFILE] [-part] [-artists <N>]
# where PLAYLISTFILE is a m3u-playlist and OUTPUTFILE is the path/name of the
# generated file... if no outputfile is given, ./playlist.html will be used by
# default
# if the -part parameter is specified, not a whole html-file will be generated,
# but only a unordered list (<ul>) containing the titles
# if -artists is specified, all artists present with more than N titles in the
# playlist will be sorted out and printed separatly @ the end of the file
################################################################################



##############################################################################
#	GLOBALS
##############################################################################
$COMPLETE_HTML = 1;
$BEST_ARTISTS = 0;


##############################################################################
#	MAIN PROGRAM
##############################################################################


&startup();
@songs = &readPlaylist();
&openOutputFile();
&writeFileHeader();
%artists = &writeListAndCountSongsPerArtist();
my %best_artists = &searchBestArtists(%artists);
writeBestArtists(%best_artists);
close (OUTPUT);
&writeCopyrightNote();
&writeFileFooter();



##############################################################################
#	FUNCTIONS
##############################################################################



# show program- and licensing-information, parse arguments
# INPUT:
#	none
# OUTPUT:
#	if script was called without arguments, show help-msg and exit
#	set $COMPLETE_HTML and $BEST_ARTISTS
sub startup()
{
	print "bluePlaylist $VERSION\n";
	print "(c) 2003 - Thomas Unterthiner\ngenerates HTML-files from m3u-playlists\n\n";

	print "This script is free software; you can redistribute it and/or\n";
	print "modify it under the terms of the GNU General Public License as\n";
	print "published by the Free Software Foundation; either version 2 of\n";
	print "the License, or (at your option) any later version.\n\n";

	if ($#ARGV < 0)
	{
		print "usage:\n";
		print "$0 PLAYLIST [OUTPUTFILE] [-part] [-artists <N>]\n";
		print "  PLAYLISTFILE:  m3u-playlist\n";
		print "  OUTPUTFILE:    (optional) name/path of the generated file\n";
		print "                  './playlist.html' used by default\n";
		print "  -part:         (optional) generate only partial-complete HTML-code\n";
		print "  -artists <N>:  (optional) search all artists present with N or more titles\n\n";
		exit 1;
	}	

	$COMPLETE_HTML = !(($#ARGV >= 2 && $ARGV[2] eq '-part') || $ARGV[1] eq '-part');	
	for ($i = 1; $i < $#ARGV; ++$i)
	{
		if ($ARGV[$i] eq '-artists')
		{
			$BEST_ARTISTS = $ARGV[++$i];
			last;
		}
	}
}



# read playlist
# INPUT
#	reads name/path of a m3u-playlistfile out of the first element of @ARGV
# OUTPUT
#	exit script if playlist doesn't exit
#	return array containing the filenames of all files in the playlists
#	without path or file-extension
sub readPlaylist
{
	open(INFILE, "$ARGV[0]") || die "can't open file $ARGV[0]: $!\n";
	
	my @playlist = <INFILE>;
	my $song_counter = 0;
	my @songs = ();
	foreach $entry(@playlist)
	{
		chomp $entry;
		if ( $entry =~ /^([^#].*)\.(mp3|wav|wma|ogg).?$/i)
		{
			#extract the song
			$tmp1 = rindex($entry, '/');
			$tmp2 = rindex($entry, '\\');
			$start = ($tmp1 > $tmp2 ? $tmp1 : $tmp2) + 1;
			push(@songs, lc substr($1, $start));
#			print ($title . "\n") unless (index($entry, '#') == 0);
		}
	}
	close(INFILE);
	
	print ("found " . ($#songs + 1) . " entries...\n");	
	return @songs;
}



# open the outputfile
# INPUT
#	read path of the outputfile out of the @ARGV - array
# OUTPUT
#	open outputfile with a handle named OUTFILE
# ASSUMTIONS
#	OUTFILE must be closed somewhere else!!!
sub openOutputFile
{
	if ($#ARGV >= 1 && index($ARGV[1],'-') != 0)
	{
		{open(OUTFILE, ">$ARGV[1]")}
	}
	else
	{
		print "\nno outputfile specified, using ./playlist.html...\n";
		open(OUTFILE, ">./playlist.html");
	}
}



# writes the beginning of the HTML-file making up the output
# INPUT
#	uses global $COMPLETE_HTML and $BEST_ARTISTS
# OUTPUT
#	if $COMPLETE_HTML is true, write the beginning of the html-file,
#	which basicly consists of the <head>-part.
#	if we have to write the best artists, write a local link to the later
#	set "favourite"-ancor, where then the best artists will be listed
sub writeFileHeader
{
	if ($COMPLETE_HTML)
	{
		print OUTFILE "<html>\n";
		print OUTFILE "<head>\n";
		print OUTFILE "	<title>$ARGV[0]</title>\n";
		print OUTFILE "</head>\n";
		print OUTFILE "<body>\n\n";
	}
	if ($BEST_ARTISTS > 0)
		{print OUTFILE "<a href=\"#favourite\">favourite artists</a>\n";}
}



# write the playlist to the outputfile, count how many times every single
# artist is presend in the list
# INPUT
#	takes an array with the playlist-entries, uses OUTFILE - filehandle
# OUTPUT
#	write the list to the output-file, number each song.
#	split every entry which is of the form "artist - title" and ount how 
#	many times every single artist is present in the list. Return that 
#	information in a hash with key=name of the artist, value=number of
#	songs he is present with in the list
# NOTE
#	this sub fullfills two tasks mainly for speed-reasons. This way we have
#	to iterate through the song-array only once
sub writeListAndCountSongsPerArtist
{
	my %interprets = ();
	print OUTFILE "<ol>\n";
	foreach $song (@songs)
	{
		#split to interpret - singer
		@songinfo = split(/\s+-\s+/, $song);
		$interprets{$songinfo[0]}++;
	
		print OUTFILE ("<li>$song</li>\n");
	}
	print OUTFILE "</ol>";
	
	return %interprets;
}



# extract all artists who are present with more than a given number of 
# titles in the list
# INPUT
#	uses global $BEST_ARTISTS var, which tells how many times an artist has
#	to occur in the playlist
#	needs hash of the form key=name of the artist, value=number of songs
# OUTPUT
#	return a hash of the same form as the input-hash, but containing only
#	artists with more then $BEST_ARTISTS titles in the list
sub searchBestArtists
{
	my %interprets = @_;
	if ($BEST_ARTISTS == 0)
		{return ();}

	my %best_interprets = ();
	foreach $i (keys %interprets)
	{
		if ($interprets{$i} >= $BEST_ARTISTS)
		{
			$best_interprets{$i} = $interprets{$i};
		}
	}
	return %best_interprets;
}



# write list of the best artists
# INPUT
#	hash of the form key=name of the artist, value=number of songs of only
#	the most-occuring artists
#	uses OUTFILE - filehandle
# OUTPUT
#	if $BEST_ARTISTS is 0, which means we don't have to search for 'em, leave
#	the function without doing anything.
#	Else write an unordered list with the best artists and the number of titles
#	with which they are present in the list
#	set a HTML-ancor named "#favourite" in the outputfile to find this list
#	more easily
sub writeBestArtists
{
	if ($BEST_ARTISTS == 0)
		{return;}
	my %artists = @_;
	print OUTFILE ("<a name=\"favourite\" id=\"favourite\"></a>\n");
	print OUTFILE ("<h2>Favourite Artists</h2>\n");
	print OUTFILE ("<ul>\n");
	foreach $i (sort keys %artists)
		{print OUTFILE ("<li>$i: $artists{$i} songs</li>\n");}
	print OUTFILE ("</ul>\n");

	my $counter = keys(%artists);
	print ("found $counter artists with more than $BEST_ARTISTS songs...\n");
}


# write a small Copyright-Note
# INPUT
#	uses global OUTFILE filehandle and $VERSION
# OUTPUT
#	write copyright-note
sub writeCopyrightNote()
{
	print OUTFILE "<br />\n";
	print OUTFILE "<hr />\n";
	print OUTFILE '<address style="'
		. 'font-size:xx-small; '
		. 'text-align:center; '
		. 'font-style:italic; '
		. 'color:black; '
		. 'background-color:white;'
		. "\">\n";

	print OUTFILE "generated with <b>bluePlaylist</b> $VERSION - "
		. '&copy; 2003 <a href="http://www.bluetiger.ch.vu/">'
		. "Thomas Unterthiner</a>\n";
	print OUTFILE "</address>\n";
}


# write the end of the HTML-file
# INPUT
#	uses global $COMPLETE_HTML and OUTFILE filehandle
# OUTPUT
#	writes closing <body> and <html> tags
sub writeFileFooter()
{
	if ($COMPLETE_HTML)
	{
		print OUTFILE "\n</body>\n";
		print OUTFILE "</html>\n";
	}
}
